@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 #ifndef MPD_OGG_COMMON_H
 #define MPD_OGG_COMMON_H
 
-#include "../decoder_api.h"
+#include "decoder_api.h"
 
 typedef enum _ogg_stream_type { VORBIS, FLAC } ogg_stream_type;
 
