@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,20 +17,23 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include "config.h"
 #include "decoder_list.h"
 #include "decoder_plugin.h"
 #include "utils.h"
-#include "config.h"
 #include "conf.h"
+#include "mpd_error.h"
 
 #include <glib.h>
 
 #include <string.h>
 
 extern const struct decoder_plugin mad_decoder_plugin;
+extern const struct decoder_plugin mpg123_decoder_plugin;
 extern const struct decoder_plugin vorbis_decoder_plugin;
 extern const struct decoder_plugin flac_decoder_plugin;
 extern const struct decoder_plugin oggflac_decoder_plugin;
+extern const struct decoder_plugin sndfile_decoder_plugin;
 extern const struct decoder_plugin audiofile_decoder_plugin;
 extern const struct decoder_plugin mp4ff_decoder_plugin;
 extern const struct decoder_plugin faad_decoder_plugin;
@@ -42,11 +45,15 @@ extern const struct decoder_plugin sidplay_decoder_plugin;
 extern const struct decoder_plugin wildmidi_decoder_plugin;
 extern const struct decoder_plugin fluidsynth_decoder_plugin;
 extern const struct decoder_plugin ffmpeg_decoder_plugin;
+extern const struct decoder_plugin gme_decoder_plugin;
 
-static const struct decoder_plugin *const decoder_plugins[] = {
+const struct decoder_plugin *const decoder_plugins[] = {
 #ifdef HAVE_MAD
 	&mad_decoder_plugin,
 #endif
+#ifdef HAVE_MPG123
+	&mpg123_decoder_plugin,
+#endif
 #ifdef ENABLE_VORBIS_DECODER
 	&vorbis_decoder_plugin,
 #endif
@@ -56,6 +63,9 @@ static const struct decoder_plugin *const decoder_plugins[] = {
 #ifdef HAVE_FLAC
 	&flac_decoder_plugin,
 #endif
+#ifdef ENABLE_SNDFILE
+	&sndfile_decoder_plugin,
+#endif
 #ifdef HAVE_AUDIOFILE
 	&audiofile_decoder_plugin,
 #endif
@@ -89,32 +99,51 @@ static const struct decoder_plugin *const decoder_plugins[] = {
 #ifdef HAVE_FFMPEG
 	&ffmpeg_decoder_plugin,
 #endif
+#ifdef HAVE_GME
+	&gme_decoder_plugin,
+#endif
+	NULL
 };
 
 enum {
-	num_decoder_plugins = G_N_ELEMENTS(decoder_plugins),
+	num_decoder_plugins = G_N_ELEMENTS(decoder_plugins) - 1,
 };
 
 /** which plugins have been initialized successfully? */
-static bool decoder_plugins_enabled[num_decoder_plugins];
+bool decoder_plugins_enabled[num_decoder_plugins];
+
+static unsigned
+decoder_plugin_index(const struct decoder_plugin *plugin)
+{
+	unsigned i = 0;
+
+	while (decoder_plugins[i] != plugin)
+		++i;
+
+	return i;
+}
+
+static unsigned
+decoder_plugin_next_index(const struct decoder_plugin *plugin)
+{
+	return plugin == 0
+		? 0 /* start with first plugin */
+		: decoder_plugin_index(plugin) + 1;
+}
 
 const struct decoder_plugin *
-decoder_plugin_from_suffix(const char *suffix, unsigned int next)
+decoder_plugin_from_suffix(const char *suffix,
+			   const struct decoder_plugin *plugin)
 {
-	static unsigned i = num_decoder_plugins;
-
 	if (suffix == NULL)
 		return NULL;
 
-	if (!next)
-		i = 0;
-	for (; i < num_decoder_plugins; ++i) {
-		const struct decoder_plugin *plugin = decoder_plugins[i];
+	for (unsigned i = decoder_plugin_next_index(plugin);
+	     decoder_plugins[i] != NULL; ++i) {
+		plugin = decoder_plugins[i];
 		if (decoder_plugins_enabled[i] &&
-		    stringFoundInStringArray(plugin->suffixes, suffix)) {
-			++i;
+		    decoder_plugin_supports_suffix(plugin, suffix))
 			return plugin;
-		}
 	}
 
 	return NULL;
@@ -130,10 +159,10 @@ decoder_plugin_from_mime_type(const char *mimeType, unsigned int next)
 
 	if (!next)
 		i = 0;
-	for (; i < num_decoder_plugins; ++i) {
+	for (; decoder_plugins[i] != NULL; ++i) {
 		const struct decoder_plugin *plugin = decoder_plugins[i];
 		if (decoder_plugins_enabled[i] &&
-		    stringFoundInStringArray(plugin->mime_types, mimeType)) {
+		    decoder_plugin_supports_mime_type(plugin, mimeType)) {
 			++i;
 			return plugin;
 		}
@@ -145,7 +174,7 @@ decoder_plugin_from_mime_type(const char *mimeType, unsigned int next)
 const struct decoder_plugin *
 decoder_plugin_from_name(const char *name)
 {
-	for (unsigned i = 0; i < num_decoder_plugins; ++i) {
+	for (unsigned i = 0; decoder_plugins[i] != NULL; ++i) {
 		const struct decoder_plugin *plugin = decoder_plugins[i];
 		if (decoder_plugins_enabled[i] &&
 		    strcmp(plugin->name, name) == 0)
@@ -155,27 +184,6 @@ decoder_plugin_from_name(const char *name)
 	return NULL;
 }
 
-void decoder_plugin_print_all_decoders(FILE * fp)
-{
-	for (unsigned i = 0; i < num_decoder_plugins; ++i) {
-		const struct decoder_plugin *plugin = decoder_plugins[i];
-		const char *const*suffixes;
-
-		if (!decoder_plugins_enabled[i])
-			continue;
-
-		fprintf(fp, "[%s]", plugin->name);
-
-		for (suffixes = plugin->suffixes;
-		     suffixes != NULL && *suffixes != NULL;
-		     ++suffixes) {
-			fprintf(fp, " %s", *suffixes);
-		}
-
-		fprintf(fp, "\n");
-	}
-}
-
 /**
  * Find the "decoder" configuration block for the specified plugin.
  *
@@ -191,8 +199,8 @@ decoder_plugin_config(const char *plugin_name)
 		const char *name =
 			config_get_block_string(param, "plugin", NULL);
 		if (name == NULL)
-			g_error("decoder configuration without 'plugin' name in line %d",
-				param->line);
+			MPD_ERROR("decoder configuration without 'plugin' name in line %d",
+				  param->line);
 
 		if (strcmp(name, plugin_name) == 0)
 			return param;
@@ -203,7 +211,7 @@ decoder_plugin_config(const char *plugin_name)
 
 void decoder_plugin_init_all(void)
 {
-	for (unsigned i = 0; i < num_decoder_plugins; ++i) {
+	for (unsigned i = 0; decoder_plugins[i] != NULL; ++i) {
 		const struct decoder_plugin *plugin = decoder_plugins[i];
 		const struct config_param *param =
 			decoder_plugin_config(plugin->name);
@@ -219,7 +227,7 @@ void decoder_plugin_init_all(void)
 
 void decoder_plugin_deinit_all(void)
 {
-	for (unsigned i = 0; i < num_decoder_plugins; ++i) {
+	for (unsigned i = 0; decoder_plugins[i] != NULL; ++i) {
 		const struct decoder_plugin *plugin = decoder_plugins[i];
 
 		if (decoder_plugins_enabled[i])
