@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,6 @@
 #include "queue.h"
 
 #include <stdbool.h>
-#include <stdio.h>
 
 #define PLAYLIST_COMMENT	'#'
 
@@ -82,21 +81,16 @@ struct playlist {
 	 * This variable is only valid if #playing is true.
 	 */
 	int queued;
-
-	/**
-	 * This timer tracks the time elapsed since the last "prev"
-	 * command.  If that is less than one second ago, "prev" jumps
-	 * to the previous song instead of rewinding the current song.
-	 */
-	GTimer *prev_elapsed;
 };
 
 /** the global playlist object */
 extern struct playlist g_playlist;
 
-void initPlaylist(void);
+void
+playlist_global_init(void);
 
-void finishPlaylist(void);
+void
+playlist_global_finish(void);
 
 void
 playlist_init(struct playlist *playlist);
@@ -116,11 +110,8 @@ playlist_get_queue(const struct playlist *playlist)
 	return &playlist->queue;
 }
 
-void readPlaylistState(FILE *);
-
-void savePlaylistState(FILE *);
-
-void clearPlaylist(struct playlist *playlist);
+void
+playlist_clear(struct playlist *playlist);
 
 #ifndef WIN32
 /**
@@ -133,90 +124,111 @@ playlist_append_file(struct playlist *playlist, const char *path, int uid,
 #endif
 
 enum playlist_result
-addToPlaylist(struct playlist *playlist, const char *file, unsigned *added_id);
+playlist_append_uri(struct playlist *playlist, const char *file,
+		    unsigned *added_id);
 
 enum playlist_result
-addSongToPlaylist(struct playlist *playlist,
+playlist_append_song(struct playlist *playlist,
 		  struct song *song, unsigned *added_id);
 
 enum playlist_result
-deleteFromPlaylist(struct playlist *playlist, unsigned song);
+playlist_delete(struct playlist *playlist, unsigned song);
+
+/**
+ * Deletes a range of songs from the playlist.
+ *
+ * @param start the position of the first song to delete
+ * @param end the position after the last song to delete
+ */
+enum playlist_result
+playlist_delete_range(struct playlist *playlist, unsigned start, unsigned end);
 
 enum playlist_result
-deleteFromPlaylistById(struct playlist *playlist, unsigned song);
-
-void stopPlaylist(struct playlist *playlist);
-
-enum playlist_result
-playPlaylist(struct playlist *playlist, int song);
-
-enum playlist_result
-playPlaylistById(struct playlist *playlist, int song);
-
-void nextSongInPlaylist(struct playlist *playlist);
-
-void syncPlayerAndPlaylist(struct playlist *playlist);
-
-void previousSongInPlaylist(struct playlist *playlist);
-
-void shufflePlaylist(struct playlist *playlist, unsigned start, unsigned end);
+playlist_delete_id(struct playlist *playlist, unsigned song);
 
 void
-deleteASongFromPlaylist(struct playlist *playlist, const struct song *song);
+playlist_stop(struct playlist *playlist);
 
 enum playlist_result
-moveSongRangeInPlaylist(struct playlist *playlist, unsigned start, unsigned end, int to);
+playlist_play(struct playlist *playlist, int song);
 
 enum playlist_result
-moveSongInPlaylistById(struct playlist *playlist, unsigned id, int to);
+playlist_play_id(struct playlist *playlist, int song);
+
+void
+playlist_next(struct playlist *playlist);
+
+void
+playlist_sync(struct playlist *playlist);
+
+void
+playlist_previous(struct playlist *playlist);
+
+void
+playlist_shuffle(struct playlist *playlist, unsigned start, unsigned end);
+
+void
+playlist_delete_song(struct playlist *playlist, const struct song *song);
 
 enum playlist_result
-swapSongsInPlaylist(struct playlist *playlist, unsigned song1, unsigned song2);
+playlist_move_range(struct playlist *playlist, unsigned start, unsigned end, int to);
 
 enum playlist_result
-swapSongsInPlaylistById(struct playlist *playlist, unsigned id1, unsigned id2);
+playlist_move_id(struct playlist *playlist, unsigned id, int to);
+
+enum playlist_result
+playlist_swap_songs(struct playlist *playlist, unsigned song1, unsigned song2);
+
+enum playlist_result
+playlist_swap_songs_id(struct playlist *playlist, unsigned id1, unsigned id2);
 
 bool
-getPlaylistRepeatStatus(const struct playlist *playlist);
+playlist_get_repeat(const struct playlist *playlist);
 
-void setPlaylistRepeatStatus(struct playlist *playlist, bool status);
+void
+playlist_set_repeat(struct playlist *playlist, bool status);
 
 bool
-getPlaylistRandomStatus(const struct playlist *playlist);
+playlist_get_random(const struct playlist *playlist);
 
-void setPlaylistRandomStatus(struct playlist *playlist, bool status);
+void
+playlist_set_random(struct playlist *playlist, bool status);
 
 bool
-getPlaylistSingleStatus(const struct playlist *playlist);
+playlist_get_single(const struct playlist *playlist);
 
-void setPlaylistSingleStatus(struct playlist *playlist, bool status);
+void
+playlist_set_single(struct playlist *playlist, bool status);
 
 bool
-getPlaylistConsumeStatus(const struct playlist *playlist);
+playlist_get_consume(const struct playlist *playlist);
 
-void setPlaylistConsumeStatus(struct playlist *playlist, bool status);
+void
+playlist_set_consume(struct playlist *playlist, bool status);
 
-int getPlaylistCurrentSong(const struct playlist *playlist);
+int
+playlist_get_current_song(const struct playlist *playlist);
 
-int getPlaylistNextSong(const struct playlist *playlist);
+int
+playlist_get_next_song(const struct playlist *playlist);
 
 unsigned
-getPlaylistSongId(const struct playlist *playlist, unsigned song);
+playlist_get_song_id(const struct playlist *playlist, unsigned song);
 
-int getPlaylistLength(const struct playlist *playlist);
+int
+playlist_get_length(const struct playlist *playlist);
 
 unsigned long
-getPlaylistVersion(const struct playlist *playlist);
+playlist_get_version(const struct playlist *playlist);
 
 enum playlist_result
-seekSongInPlaylist(struct playlist *playlist, unsigned song, float seek_time);
+playlist_seek_song(struct playlist *playlist, unsigned song, float seek_time);
 
 enum playlist_result
-seekSongInPlaylistById(struct playlist *playlist,
+playlist_seek_song_id(struct playlist *playlist,
 		       unsigned id, float seek_time);
 
-void playlistVersionChange(struct playlist *playlist);
-
-int is_valid_playlist_name(const char *utf8path);
+void
+playlist_increment_version_all(struct playlist *playlist);
 
 #endif
